import pytest
from lxml import etree

from verbecc import Conjugator
from verbecc.tense_template import TenseTemplate

cg = Conjugator(lang="ca")


# TODO: Fix all of these missing templates
@pytest.mark.skip("known failure")
def test_all_verbs_have_templates():
    """Have not finished adding templates for all verbs, so this should fail"""
    verbs = cg.get_verbs()
    assert len(verbs) == 8616
    template_names = cg.get_template_names()
    assert len(template_names) == 50
    missing_verbs = []
    missing_templates = set()
    for verb in verbs:
        if verb.template not in template_names:
            missing_templates.add(verb.template)
            missing_verbs.append(verb)
    assert len(missing_templates) == 0
    # assert len(missing_verbs) == 0


def test_find_verb_by_infinitive():
    v = cg.find_verb_by_infinitive("abandonar")
    assert v.infinitive == "abandonar"
    assert v.template == "cant:ar"


@pytest.mark.parametrize(
    "infinitive,mood,tense,alternate,gender,expected_result",
    [
        (
            "ser",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo sóc",
                "tu ets",
                "ell és",
                "nosaltres som",
                "vosaltres sou",
                "ells són",
            ],
        ),
        (
            "ser",
            "indicatiu",
            "imperfet",
            False,
            "m",
            [
                "jo era",
                "tu eres",
                "ell era",
                "nosaltres érem",
                "vosaltres éreu",
                "ells eren",
            ],
        ),
        (
            "ser",
            "indicatiu",
            "passat-simple",
            False,
            "m",
            [
                "jo fui",
                "tu fores",
                "ell fou",
                "nosaltres fórem",
                "vosaltres fóreu",
                "ells foren",
            ],
        ),
        (
            "ser",
            "indicatiu",
            "futur",
            False,
            "m",
            [
                "jo seré",
                "tu seràs",
                "ell serà",
                "nosaltres serem",
                "vosaltres sereu",
                "ells seran",
            ],
        ),
        (
            "ser",
            "subjuntiu",
            "present",
            False,
            "m",
            [
                "jo sigui",
                "tu siguis",
                "ell sigui",
                "nosaltres siguem",
                "vosaltres sigueu",
                "ells siguin",
            ],
        ),
        (
            "ser",
            "subjuntiu",
            "imperfet",
            False,
            "m",
            [
                "jo fos",
                "tu fossis",
                "ell fos",
                "nosaltres fóssim",
                "vosaltres fóssiu",
                "ells fossin",
            ],
        ),
        (
            "ser",
            "condicional",
            "present",
            False,
            "m",
            [
                "jo seria",
                "tu series",
                "ell seria",
                "nosaltres seríem",
                "vosaltres seríeu",
                "ells serien",
            ],
        ),
        (
            "ser",
            "imperatiu",
            "imperatiu-present",
            False,
            "m",
            ["sigues", "sigui", "siguem", "sigueu", "siguin"],
        ),
        (
            "parlar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo parlo",
                "tu parles",
                "ell parla",
                "nosaltres parlem",
                "vosaltres parleu",
                "ells parlen",
            ],
        ),
        (
            "parlar",
            "indicatiu",
            "imperfet",
            False,
            "m",
            [
                "jo parlava",
                "tu parlaves",
                "ell parlava",
                "nosaltres parlàvem",
                "vosaltres parlàveu",
                "ells parlaven",
            ],
        ),
        (
            "parlar",
            "indicatiu",
            "passat-simple",
            False,
            "m",
            [
                "jo parlí",
                "tu parlares",
                "ell parlà",
                "nosaltres parlàrem",
                "vosaltres parlàreu",
                "ells parlaren",
            ],
        ),
        (
            "parlar",
            "indicatiu",
            "futur",
            False,
            "m",
            [
                "jo parlaré",
                "tu parlaràs",
                "ell parlarà",
                "nosaltres parlarem",
                "vosaltres parlareu",
                "ells parlaran",
            ],
        ),
        (
            "parlar",
            "subjuntiu",
            "present",
            False,
            "m",
            [
                "jo parli",
                "tu parlis",
                "ell parli",
                "nosaltres parlem",
                "vosaltres parleu",
                "ells parlin",
            ],
        ),
        (
            "parlar",
            "subjuntiu",
            "imperfet",
            False,
            "m",
            [
                "jo parlés",
                "tu parlessis",
                "ell parlés",
                "nosaltres parléssim",
                "vosaltres parléssiu",
                "ells parlessin",
            ],
        ),
        (
            "parlar",
            "condicional",
            "present",
            False,
            "m",
            [
                "jo parlaria",
                "tu parlaries",
                "ell parlaria",
                "nosaltres parlaríem",
                "vosaltres parlaríeu",
                "ells parlarien",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo estic",
                "tu estàs",
                "ell està",
                "nosaltres estem",
                "vosaltres esteu",
                "ells estan",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "imperfet",
            False,
            "m",
            [
                "jo estava",
                "tu estaves",
                "ell estava",
                "nosaltres estàvem",
                "vosaltres estàveu",
                "ells estaven",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "passat-simple",
            False,
            "m",
            [
                "jo estiguí",
                "tu estigueres",
                "ell estigué",
                "nosaltres estiguérem",
                "vosaltres estiguéreu",
                "ells estigueren",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "futur",
            False,
            "m",
            [
                "jo estaré",
                "tu estaràs",
                "ell estarà",
                "nosaltres estarem",
                "vosaltres estareu",
                "ells estaran",
            ],
        ),
        (
            "estar",
            "subjuntiu",
            "present",
            False,
            "m",
            [
                "jo estigui",
                "tu estiguis",
                "ell estigui",
                "nosaltres estiguem",
                "vosaltres estigueu",
                "ells estiguin",
            ],
        ),
        (
            "estar",
            "subjuntiu",
            "imperfet",
            False,
            "m",
            [
                "jo estigués",
                "tu estiguessis",
                "ell estigués",
                "nosaltres estiguéssim",
                "vosaltres estiguéssiu",
                "ells estiguessin",
            ],
        ),
        (
            "estar",
            "condicional",
            "present",
            False,
            "m",
            [
                "jo estaria",
                "tu estaries",
                "ell estaria",
                "nosaltres estaríem",
                "vosaltres estaríeu",
                "ells estarien",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo he",
                "tu has",
                "ell ha",
                "nosaltres havem",
                "vosaltres haveu",
                "ells han",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "imperfet",
            False,
            "m",
            [
                "jo havia",
                "tu havies",
                "ell havia",
                "nosaltres havíem",
                "vosaltres havíeu",
                "ells havien",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "passat-simple",
            False,
            "m",
            [
                "jo haguí",
                "tu hagueres",
                "ell hagué",
                "nosaltres haguérem",
                "vosaltres haguéreu",
                "ells hagueren",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "futur",
            False,
            "m",
            [
                "jo hauré",
                "tu hauràs",
                "ell haurà",
                "nosaltres haurem",
                "vosaltres haureu",
                "ells hauran",
            ],
        ),
        (
            "haver",
            "condicional",
            "present",
            False,
            "m",
            [
                "jo hauria",
                "tu hauries",
                "ell hauria",
                "nosaltres hauríem",
                "vosaltres hauríeu",
                "ells haurien",
            ],
        ),
        (
            "haver",
            "subjuntiu",
            "present",
            False,
            "m",
            [
                "jo hagi",
                "tu hagis",
                "ell hagi",
                "nosaltres hàgim",
                "vosaltres hàgiu",
                "ells hagin",
            ],
        ),
        (
            "tenir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo tinc",
                "tu tens",
                "ell té",
                "nosaltres tenim",
                "vosaltres teniu",
                "ells tenen",
            ],
        ),
        (
            "fer",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo faig",
                "tu fas",
                "ell fa",
                "nosaltres fem",
                "vosaltres feu",
                "ells fan",
            ],
        ),
        (
            "fer",
            "indicatiu",
            "imperfet",
            False,
            "m",
            [
                "jo feia",
                "tu feies",
                "ell feia",
                "nosaltres fèiem",
                "vosaltres fèieu",
                "ells feien",
            ],
        ),
        (
            "servir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo serveixo",
                "tu serveixes",
                "ell serveix",
                "nosaltres servim",
                "vosaltres serviu",
                "ells serveixen",
            ],
        ),
        (
            "veure",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo veig",
                "tu veus",
                "ell veu",
                "nosaltres veiem",
                "vosaltres veieu",
                "ells veuen",
            ],
        ),
        (
            "abandonar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo abandono",
                "tu abandones",
                "ell abandona",
                "nosaltres abandonem",
                "vosaltres abandoneu",
                "ells abandonen",
            ],
        ),
        (
            "rebre",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo rebo",
                "tu reps",
                "ell rep",
                "nosaltres rebem",
                "vosaltres rebeu",
                "ells reben",
            ],
        ),
        (
            "cabre",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo cabo",
                "tu caps",
                "ell cap",
                "nosaltres cabem",
                "vosaltres cabeu",
                "ells caben",
            ],
        ),
        (
            "començar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo començo",
                "tu comences",
                "ell comença",
                "nosaltres comencem",
                "vosaltres comenceu",
                "ells comencen",
            ],
        ),
        (
            "enaiguar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo enaiguo",
                "tu enaigües",
                "ell enaigua",
                "nosaltres enaigüem",
                "vosaltres enaigüeu",
                "ells enaigüen",
            ],
        ),
        (
            "envejar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo envejo",
                "tu enveges",
                "ell enveja",
                "nosaltres envegem",
                "vosaltres envegeu",
                "ells envegen",
            ],
        ),
        (
            "esglaiar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo esglaio",
                "tu esglaies",
                "ell esglaia",
                "nosaltres esglaiem",
                "vosaltres esglaieu",
                "ells esglaien",
            ],
        ),
        (
            "trencar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo trenco",
                "tu trenques",
                "ell trenca",
                "nosaltres trenquem",
                "vosaltres trenqueu",
                "ells trenquen",
            ],
        ),
        (
            "perdre",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo perdo",
                "tu perds",
                "ell perd",
                "nosaltres perdem",
                "vosaltres perdeu",
                "ells perden",
            ],
        ),
        (
            "caure",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo caic",
                "tu caus",
                "ell cau",
                "nosaltres caiem",
                "vosaltres caieu",
                "ells cauen",
            ],
        ),
        (
            "collar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo collo",
                "tu colles",
                "ell colla",
                "nosaltres collem",
                "vosaltres colleu",
                "ells collen",
            ],
        ),
        (
            "collir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo cullo",
                "tu culls",
                "ell cull",
                "nosaltres collim",
                "vosaltres colliu",
                "ells cullen",
            ],
        ),
        (
            "dormir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo dormo",
                "tu dorms",
                "ell dorm",
                "nosaltres dormim",
                "vosaltres dormiu",
                "ells dormen",
            ],
        ),
        (
            "tossir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo tusso",
                "tu tusses",
                "ell tus",
                "nosaltres tossim",
                "vosaltres tossiu",
                "ells tussen",
            ],
        ),
        (
            "collir",
            "subjuntiu",
            "present",
            False,
            "m",
            [
                "jo culli",
                "tu cullis",
                "ell culli",
                "nosaltres collim",
                "vosaltres colliu",
                "ells cullin",
            ],
        ),
        (
            "dormir",
            "subjuntiu",
            "present",
            False,
            "m",
            [
                "jo dormi",
                "tu dormis",
                "ell dormi",
                "nosaltres dormim",
                "vosaltres dormiu",
                "ells dormin",
            ],
        ),
        (
            "tossir",
            "subjuntiu",
            "present",
            False,
            "m",
            [
                "jo tussi",
                "tu tussis",
                "ell tussi",
                "nosaltres tossim",
                "vosaltres tossiu",
                "ells tussin",
            ],
        ),
        (
            "dir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo dic",
                "tu dius",
                "ell diu",
                "nosaltres diem",
                "vosaltres dieu",
                "ells diuen",
            ],
        ),
        (
            "anar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo vaig",
                "tu vas",
                "ell va",
                "nosaltres anem",
                "vosaltres aneu",
                "ells van",
            ],
        ),
        (
            "poder",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo puc",
                "tu pots",
                "ell pot",
                "nosaltres podem",
                "vosaltres podeu",
                "ells poden",
            ],
        ),
        (
            "arribar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo arribo",
                "tu arribes",
                "ell arriba",
                "nosaltres arribem",
                "vosaltres arribeu",
                "ells arriben",
            ],
        ),
        (
            "trobar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo trobo",
                "tu trobes",
                "ell troba",
                "nosaltres trobem",
                "vosaltres trobeu",
                "ells troben",
            ],
        ),
        (
            "partir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo parteixo",
                "tu parteixes",
                "ell parteix",
                "nosaltres partim",
                "vosaltres partiu",
                "ells parteixen",
            ],
        ),
        (
            "morir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo moro",
                "tu mors",
                "ell mor",
                "nosaltres morim",
                "vosaltres moriu",
                "ells moren",
            ],
        ),
        (
            "eixir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo ixo",
                "tu ixes",
                "ell ix",
                "nosaltres eixim",
                "vosaltres eixiu",
                "ells ixen",
            ],
        ),
        (
            "jaure",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo jec",
                "tu jeus",
                "ell jeu",
                "nosaltres jaiem",
                "vosaltres jaieu",
                "ells jeuen",
            ],
        ),
        (
            "jeure",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo jec",
                "tu jeus",
                "ell jeu",
                "nosaltres jaiem",
                "vosaltres jaieu",
                "ells jeuen",
            ],
        ),
        (
            "pagar",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo pago",
                "tu pagues",
                "ell paga",
                "nosaltres paguem",
                "vosaltres pagueu",
                "ells paguen",
            ],
        ),
        (
            "mentir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo menteixo",
                "tu menteixes",
                "ell menteix",
                "nosaltres mentim",
                "vosaltres mentiu",
                "ells menteixen",
            ],
        ),
        (
            "desmentir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo desmenteixo",
                "tu desmenteixes",
                "ell desmenteix",
                "nosaltres desmentim",
                "vosaltres desmentiu",
                "ells desmenteixen",
            ],
        ),
        (
            "nàixer",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo naixo",
                "tu naixes",
                "ell naix",
                "nosaltres naixem",
                "vosaltres naixeu",
                "ells naixen",
            ],
        ),
        (
            "néixer",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo naixo",
                "tu naixes",
                "ell naix",
                "nosaltres naixem",
                "vosaltres naixeu",
                "ells naixen",
            ],
        ),
        (
            "obrir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo obro",
                "tu obres",
                "ell obre",
                "nosaltres obrim",
                "vosaltres obriu",
                "ells obren",
            ],
        ),
        (
            "riure",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo ric",
                "tu rius",
                "ell riu",
                "nosaltres riem",
                "vosaltres rieu",
                "ells riuen",
            ],
        ),
        (
            "pertànyer",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo pertanyo",
                "tu pertanys",
                "ell pertany",
                "nosaltres pertanyem",
                "vosaltres pertanyeu",
                "ells pertanyen",
            ],
        ),
        (
            "pertànyer",
            "indicatiu",
            "passat-simple",
            False,
            "m",
            [
                "jo pertanyí",
                "tu pertanyeres",
                "ell pertanyé",
                "nosaltres pertanyérem",
                "vosaltres pertanyéreu",
                "ells pertanyeren",
            ],
        ),
        (
            "pertànyer",
            "indicatiu",
            "passat-simple",
            True,
            "m",
            [
                "jo pertanguí",
                "tu pertangueres",
                "ell pertangué",
                "nosaltres pertanguérem",
                "vosaltres pertanguéreu",
                "ells pertanguéren",
            ],
        ),
        (
            "fènyer",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo fenyo",
                "tu fenys",
                "ell feny",
                "nosaltres fenyem",
                "vosaltres fenyeu",
                "ells fenyen",
            ],
        ),
        (
            "córrer",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo corro",
                "tu corres",
                "ell corre",
                "nosaltres correm",
                "vosaltres correu",
                "ells corren",
            ],
        ),
        (
            "transcórrer",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo transcorro",
                "tu transcorres",
                "ell transcorre",
                "nosaltres transcorrem",
                "vosaltres transcorreu",
                "ells transcorren",
            ],
        ),
        (
            "imprimir",
            "indicatiu",
            "present",
            False,
            "m",
            [
                "jo imprimeixo",
                "tu imprimeixes",
                "ell imprimeix",
                "nosaltres imprimim",
                "vosaltres imprimiu",
                "ells imprimeixen",
            ],
        ),
        (
            "imprimir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo imprimeixo",
                "tu imprimeixes",
                "ella imprimeix",
                "nosaltres imprimim",
                "vosaltres imprimiu",
                "elles imprimeixen",
            ],
        ),
        (
            "vindre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo vindo",
                "tu vinds",
                "ella vind",
                "nosaltres vindem",
                "vosaltres vindeu",
                "elles vinden",
            ],
        ),
        (
            "tòrcer",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo torço",
                "tu torces",
                "ella torça",
                "nosaltres torcem",
                "vosaltres torceu",
                "elles torcen",
            ],
        ),
        (
            "caldre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "-",
                "-",
                "ella cal",
                "-",
                "-",
                "elles calen",
            ],
        ),
        (
            "obeir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo obeeixo",
                "tu obeeixes",
                "ella obeeix",
                "nosaltres obeïm",
                "vosaltres obeïu",
                "elles obeeixen",
            ],
        ),
        (
            "beneir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo beneeixo",
                "tu beneeixes",
                "ella beneeix",
                "nosaltres beneïm",
                "vosaltres beneïu",
                "elles beneeixen",
            ],
        ),
        (
            "coldre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo colc",
                "tu cols",
                "ella col",
                "nosaltres colem",
                "vosaltres coleu",
                "elles colen",
            ],
        ),
        (
            "dissoldre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo dissolc",
                "tu dissols",
                "ella dissol",
                "nosaltres dissolem",
                "vosaltres dissoleu",
                "elles dissolen",
            ],
        ),
        (
            "percudir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo percudeixo",
                "tu percudeixes",
                "ella percudeix",
                "nosaltres percudim",
                "vosaltres percudiu",
                "elles percudeixen",
            ],
        ),
        (
            "percudir",
            "indicatiu",
            "present",
            True,
            "f",
            [
                "jo percudo",
                "tu percuts",
                "ella percut",
                "nosaltres percudim",
                "vosaltres percudiu",
                "elles percuden",
            ],
        ),
        (
            "permetre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo permeto",
                "tu permets",
                "ella permet",
                "nosaltres permetem",
                "vosaltres permeteu",
                "elles permeten",
            ],
        ),
        (
            "desconèixer",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo desconec",
                "tu desconeixes",
                "ella desconeix",
                "nosaltres desconeixem",
                "vosaltres desconeixeu",
                "elles desconeixen",
            ],
        ),
        (
            "desconèixer",
            "indicatiu",
            "passat-simple",
            False,
            "f",
            [
                "jo desconeguí",
                "tu desconegueres",
                "ella desconegué",
                "nosaltres desconeguérem",
                "vosaltres desconeguéreu",
                "elles desconegueren",
            ],
        ),
        (
            "parèixer",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo parec",
                "tu pareixes",
                "ella pareix",
                "nosaltres pareixem",
                "vosaltres pareixeu",
                "elles pareixen",
            ],
        ),
        (
            "expendre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo expenc",
                "tu expens",
                "ella expèn",
                "nosaltres expenem",
                "vosaltres expeneu",
                "elles expenen",
            ],
        ),
        (
            "contendre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo contenc",
                "tu contens",
                "ella contén",
                "nosaltres contenem",
                "vosaltres conteneu",
                "elles contenen",
            ],
        ),
        (
            "atènyer",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo atenyo",
                "tu atenys",
                "ella ateny",
                "nosaltres atenyem",
                "vosaltres atenyeu",
                "elles atenyen",
            ],
        ),
        (
            "descloure",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo descloc",
                "tu desclous",
                "ella desclou",
                "nosaltres descloem",
                "vosaltres descloeu",
                "elles desclouen",
            ],
        ),
        (
            "descloure",
            "particip",
            "particip",
            False,
            "f",
            ["desclòs", "desclosa", "desclosos", "descloses"],
        ),
        (
            "cloure",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo cloc",
                "tu clous",
                "ella clou",
                "nosaltres cloem",
                "vosaltres cloeu",
                "elles clouen",
            ],
        ),
        (
            "abreviar",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo abrevio",
                "tu abrevies",
                "ella abrevia",
                "nosaltres abreviem",
                "vosaltres abrevieu",
                "elles abrevien",
            ],
        ),
        (
            "abreviar",
            "subjuntiu",
            "present",
            False,
            "f",
            [
                "jo abreviï",
                "tu abreviïs",
                "ella abreviï",
                "nosaltres abreviem",
                "vosaltres abrevieu",
                "elles abreviïn",
            ],
        ),
        (
            "abreviar",
            "imperatiu",
            "imperatiu-present",
            False,
            "m",
            ["abrevia", "abreviï", "abreviem", "abrevieu", "abreviïn"],
        ),
        (
            "compondre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo componc",
                "tu compons",
                "ella compon",
                "nosaltres componem",
                "vosaltres componeu",
                "elles componen",
            ],
        ),
        (
            "contendre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo contenc",
                "tu contens",
                "ella contén",
                "nosaltres contenem",
                "vosaltres conteneu",
                "elles contenen",
            ],
        ),
        (
            "complir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo compleixo",
                "tu compleixes",
                "ella compleix",
                "nosaltres complim",
                "vosaltres compliu",
                "elles compleixen",
            ],
        ),
        (
            "establir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo estableixo",
                "tu estableixes",
                "ella estableix",
                "nosaltres establim",
                "vosaltres establiu",
                "elles estableixen",
            ],
        ),
        (
            "complaure",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo complac",
                "tu complaus",
                "ella complau",
                "nosaltres complaem",
                "vosaltres complaeu",
                "elles complauen",
            ],
        ),
        (
            "desplaure",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo desplac",
                "tu desplaus",
                "ella desplau",
                "nosaltres desplaem",
                "vosaltres desplaeu",
                "elles desplauen",
            ],
        ),
        (
            "complaure",
            "particip",
            "particip",
            False,
            "f",
            ["complagut", "complaguda", "complaguts", "complagudes"],
        ),
        (
            "acréixer",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo acreixo",
                "tu acreixes",
                "ella acreix",
                "nosaltres acreixem",
                "vosaltres acreixeu",
                "elles acreixen",
            ],
        ),
        (
            "acréixer",
            "indicatiu",
            "passat-simple",
            False,
            "f",
            [
                "jo acreixí",
                "tu acreixeres",
                "ella acreixé",
                "nosaltres acreixérem",
                "vosaltres acreixéreu",
                "elles acreixeren",
            ],
        ),
        (
            "acréixer",
            "indicatiu",
            "passat-simple",
            True,
            "f",
            [
                "jo acresquí",
                "tu acresqueres",
                "ella acresqué",
                "nosaltres acresquérem",
                "vosaltres acresquéreu",
                "elles acresqueren",
            ],
        ),
        (
            "acréixer",
            "subjuntiu",
            "imperfet",
            False,
            "f",
            [
                "jo acreixés",
                "tu acreixessis",
                "ella acreixés",
                "nosaltres acreixéssim",
                "vosaltres acreixéssiu",
                "elles acreixessin",
            ],
        ),
        (
            "acréixer",
            "subjuntiu",
            "imperfet",
            True,
            "f",
            [
                "jo acresqués",
                "tu acresquessis",
                "ella acresqués",
                "nosaltres acresquéssim",
                "vosaltres acresquéssiu",
                "elles acresquessin",
            ],
        ),
        (
            "merèixer",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo mereixo",
                "tu mereixes",
                "ella mereix",
                "nosaltres mereixem",
                "vosaltres mereixeu",
                "elles mereixen",
            ],
        ),
        (
            "confondre",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo confonc",
                "tu confons",
                "ella confon",
                "nosaltres confonem",
                "vosaltres confoneu",
                "elles confonen",
            ],
        ),
        (
            "consternar",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo consterno",
                "tu consternes",
                "ella consterna",
                "nosaltres consternem",
                "vosaltres consterneu",
                "elles consternen",
            ],
        ),
        (
            "contradir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo contradic",
                "tu contradius",
                "ella contradiu",
                "nosaltres contradiem",
                "vosaltres contradieu",
                "elles contradiuen",
            ],
        ),
        (
            "contradir",
            "particip",
            "particip",
            False,
            "f",
            ["contradit", "contradida", "contradits", "contradides"],
        ),
        (
            "dir",
            "particip",
            "particip",
            False,
            "f",
            ["dit", "dita", "dits", "dites"],
        ),
        (
            "creure",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo crec",
                "tu creus",
                "ella creu",
                "nosaltres creiem",
                "vosaltres creieu",
                "elles creuen",
            ],
        ),
        (
            "creure",
            "indicatiu",
            "futur",
            False,
            "f",
            [
                "jo creuré",
                "tu creuràs",
                "ella creurà",
                "nosaltres creurem",
                "vosaltres creureu",
                "elles creuran",
            ],
        ),
        (
            "dur",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo duc",
                "tu duus",
                "ella duu",
                "nosaltres duem",
                "vosaltres dueu",
                "elles duen",
            ],
        ),
        (
            "dur",
            "indicatiu",
            "present",
            True,
            "f",
            [
                "jo duc",
                "tu dus",
                "ella du",
                "nosaltres duem",
                "vosaltres dueu",
                "elles duen",
            ],
        ),
        (
            "dur",
            "imperatiu",
            "imperatiu-present",
            False,
            "m",
            ["duu", "dugui", "duguem", "dueu", "duguin"],
        ),
        (
            "dur",
            "imperatiu",
            "imperatiu-present",
            True,
            "m",
            ["du", "dugui", "duguem", "dueu", "duguin"],
        ),
        (
            "descosir",
            "indicatiu",
            "present",
            True,
            "f",
            [
                "jo descuso",
                "tu descuses",
                "ella descús",
                "nosaltres descosim",
                "vosaltres descosiu",
                "elles descusen",
            ],
        ),
        (
            "descosir",
            "subjuntiu",
            "present",
            True,
            "f",
            [
                "jo descusi",
                "tu descusis",
                "ella descusi",
                "nosaltres descosim",
                "vosaltres descosiu",
                "elles descusin",
            ],
        ),
        (
            "descosir",
            "imperatiu",
            "imperatiu-present",
            False,
            "m",
            ["descús", "descusi", "descosim", "descosiu", "descusin"],
        ),
        (
            "valer",
            "indicatiu",
            "present",
            True,
            "f",
            [
                "jo valc",
                "tu vals",
                "ella val",
                "nosaltres valem",
                "vosaltres valeu",
                "elles valen",
            ],
        ),
        (
            "valer",
            "indicatiu",
            "passat-simple",
            True,
            "f",
            [
                "jo valguí",
                "tu valgueres",
                "ella valgué",
                "nosaltres valguérem",
                "vosaltres valguéreu",
                "elles valgueren",
            ],
        ),
        (
            "doldre",
            "indicatiu",
            "present",
            True,
            "f",
            [
                "jo dolc",
                "tu dols",
                "ella dol",
                "nosaltres dolem",
                "vosaltres doleu",
                "elles dolen",
            ],
        ),
        (
            "entrelluir",
            "indicatiu",
            "present",
            False,
            "f",
            [
                "jo entrelluo",
                "tu entrelluus",
                "ella entrelluu",
                "nosaltres entrelluïm",
                "vosaltres entrelluïu",
                "elles entrelluen",
            ],
        ),
        (
            "entrelluir",
            "indicatiu",
            "present",
            True,
            "f",
            [
                "jo entrelluo",
                "tu entrelluus",
                "ella entrellú",
                "nosaltres entrelluïm",
                "vosaltres entrelluïu",
                "elles entrelluen",
            ],
        ),
        (
            "entrelluir",
            "indicatiu",
            "imperfet",
            True,
            "f",
            [
                "jo entrelluïa",
                "tu entrelluïes",
                "ella entrelluïa",
                "nosaltres entrelluíem",
                "vosaltres entrelluíeu",
                "elles entrelluïen",
            ],
        ),
        (
            "entrelluir",
            "imperatiu",
            "imperatiu-present",
            False,
            "m",
            ["entrelluu", "entrelluï", "entrelluïm", "entrelluïu", "entrelluïn"],
        ),
        (
            "entrelluir",
            "imperatiu",
            "imperatiu-present",
            True,
            "m",
            ["entrellú", "entrelluï", "entrelluïm", "entrelluïu", "entrelluïn"],
        ),
    ],
)
def test_inflector_ca_conjugate_mood_tense(
    infinitive, mood, tense, alternate, gender, expected_result
):
    co = cg._inflector._get_conj_obs(infinitive)
    print("template:" + co.template.name)
    assert (
        cg.conjugate_mood_tense(infinitive, mood, tense, alternate, gender)
        == expected_result
    )


def test_inflector_ca_get_conj_obs():
    co = cg._inflector._get_conj_obs("parlar")
    assert co.verb.infinitive == "parlar"
    assert co.verb_stem == "parl"
    assert co.template.name == "cant:ar"


def test_inflector_ca_get_conj_obs_2():
    co = cg._inflector._get_conj_obs("abandonar")
    assert co.verb.infinitive == "abandonar"
    assert co.verb_stem == "abandon"
    assert co.template.name == "cant:ar"


def test_inflector_ca_get_conj_obs_3():
    co = cg._inflector._get_conj_obs("pertànyer")
    assert co.verb.infinitive == "pertànyer"
    assert (
        co.verb_stem == "pertany"
    )  # modify-stem="strip-accents" changes "pertàny" to "pertany"
    assert co.template.name == "pertàny:er"


def test_inflector_ca_get_verb_stem_from_template_name():
    verb_stem = cg._inflector._get_verb_stem_from_template_name("parlar", "cant:ar")
    assert verb_stem == "parl"


def test_inflector_ca_get_verb_stem_from_template_name_2():
    verb_stem = cg._inflector._get_verb_stem_from_template_name("abandonar", "cant:ar")
    assert verb_stem == "abandon"


def test_inflector_ca_conjugate_simple_mood_tense():
    verb_stem = "parl"
    tense_elem = etree.fromstring(
        """<present>
            <p><i>o</i></p>
            <p><i>es</i></p>
            <p><i>a</i></p>
            <p><i>em</i></p>
            <p><i>eu</i></p>
            <p><i>en</i></p>
        </present>""",
        parser=None,
    )
    tense_name = "present"
    tense_template = TenseTemplate(tense_elem)
    out = cg._inflector._conjugate_simple_mood_tense(
        verb_stem, "indicatiu", tense_template
    )
    assert len(out) == 6
    assert out == [
        "jo parlo",
        "tu parles",
        "ell parla",
        "nosaltres parlem",
        "vosaltres parleu",
        "ells parlen",
    ]


@pytest.mark.parametrize(
    "person,gender,is_reflexive,expected_result",
    [
        ("1s", "m", False, "jo"),
        ("1s", "m", True, "jo me"),
        ("2s", "m", False, "tu"),
        ("2s", "m", True, "tu te"),
        ("3s", "m", False, "ell"),
        ("3s", "m", True, "ell se"),
        ("3s", "f", False, "ella"),
        ("3s", "f", True, "ella se"),
        ("1p", "m", False, "nosaltres"),
        ("1p", "m", True, "nosaltres nos"),
        ("2p", "m", False, "vosaltres"),
        ("2p", "m", True, "vosaltres os"),
        ("3p", "m", False, "ells"),
        ("3p", "m", True, "ells se"),
        ("3p", "f", False, "elles"),
        ("3p", "f", True, "elles se"),
    ],
)
def test_inflector_ca_get_default_pronoun(
    person, gender, is_reflexive, expected_result
):
    assert (
        cg._inflector._get_default_pronoun(person, gender, is_reflexive=is_reflexive)
        == expected_result
    )
